<?php

namespace EventPortal\Tests\Unit\Agenda;

use Carbon\Carbon;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Tests\Support\Factories\AgendaFactory;
use EventPortal\Tests\Support\Factories\PathwayFactory;
use EventPortal\Tests\Support\Factories\SessionFactory;
use EventPortal\Tests\TestCase;
use Facades\EventPortal\Domain\Agenda\Agendas;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class AgendasTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function get_for_delegate()
    {
        Carbon::setTestNow(Carbon::parse('09 September 2020'));
        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();

        AgendaFactory::make(now(), $event)
            ->createPathway('Default pathway', static function (PathwayFactory $pathwayFactory) use ($delegate): void {
                $pathwayFactory->addDelegate($delegate);
                $pathwayFactory->createSession(
                    [
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ],
                    static function (SessionFactory $sessionFactory): void {
                        $sessionFactory->createVenue([
                            'name' => 'Venue one',
                        ]);

                        $sessionFactory->createSpeakerAndUser([], [
                            'first_name' => 'Speaker',
                            'last_name' => 'One',
                        ]);
                    }
                );
            });

        $data = Agendas::forDelegate($delegate, $event);

        $this->assertMatchesSnapshot($data);
    }

    /** @test */
    public function gets_multiple_days()
    {
        Carbon::setTestNow(Carbon::parse('09 September 2020'));
        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();

        AgendaFactory::make(now(), $event)
            ->createPathway('Pathway 1', static function (PathwayFactory $pathwayFactory) use ($delegate): void {
                $pathwayFactory->addDelegate($delegate);
                $pathwayFactory->createSession(
                    [
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ],
                    static function (SessionFactory $sessionFactory): void {
                        $sessionFactory->createVenue([
                            'name' => 'Venue one',
                        ]);

                        $sessionFactory->createSpeakerAndUser([], [
                            'first_name' => 'Speaker',
                            'last_name' => 'One',
                        ]);
                    }
                );
            });

        AgendaFactory::make(now()->addDay(), $event)
            ->createPathway('Pathway 1', static function (PathwayFactory $pathwayFactory) use ($delegate): void {
                $pathwayFactory->addDelegate($delegate);
                $pathwayFactory->createSession(
                    [
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session for day two',
                        'description' => 'Description for session',
                        'location' => 'Studio 2',
                    ],
                    static function (SessionFactory $sessionFactory): void {
                        $sessionFactory->createVenue([
                            'name' => 'Venue two',
                        ]);

                        $sessionFactory->createSpeakerAndUser([], [
                            'first_name' => 'Speaker',
                            'last_name' => 'Two',
                        ]);
                    }
                );
            });

        $data = Agendas::forDelegate($delegate, $event);

        $this->assertMatchesSnapshot($data);
    }

    /** @test */
    public function gets_correct_pathway_for_delegate()
    {
        Carbon::setTestNow(Carbon::parse('09 September 2020'));
        $event = factory(Event::class)->create();
        $delegateA = factory(Delegate::class)->create();
        $delegateB = factory(Delegate::class)->create();

        AgendaFactory::make(now(), $event)
            ->createPathway('Pathway 1', static function (PathwayFactory $pathwayFactory) use ($delegateA): void {
                $pathwayFactory->addDelegate($delegateA);
                $pathwayFactory->createSession(
                    [
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ],
                    static function (SessionFactory $sessionFactory): void {
                        $sessionFactory->createVenue([
                            'name' => 'Venue one',
                        ]);

                        $sessionFactory->createSpeakerAndUser([], [
                            'first_name' => 'Speaker',
                            'last_name' => 'One',
                        ]);
                    }
                );
            });

        AgendaFactory::make(now()->addDay(), $event)
            ->createPathway('Pathway 1', static function (PathwayFactory $pathwayFactory) use ($delegateB): void {
                $pathwayFactory->addDelegate($delegateB);
                $pathwayFactory->createSession(
                    [
                        'starting_at' => now(),
                        'ending_at' => now()->addHour(),
                        'name' => 'Session for day two',
                        'description' => 'Description for session',
                        'location' => 'Studio 2',
                    ],
                    static function (SessionFactory $sessionFactory): void {
                        $sessionFactory->createVenue([
                            'name' => 'Venue two',
                        ]);

                        $sessionFactory->createSpeakerAndUser([], [
                            'first_name' => 'Speaker',
                            'last_name' => 'Two',
                        ]);
                    }
                );
            });

        $dataForDelegateA = Agendas::forDelegate($delegateA, $event);
        $this->assertMatchesSnapshot($dataForDelegateA);

        $dataForDelegateB = Agendas::forDelegate($delegateB, $event);
        $this->assertMatchesSnapshot($dataForDelegateB);
    }
}
